package com.lyg.framework.log;

import com.lyg.common.util.IpUtil;
import com.lyg.framework.service.IOpLogService;
import com.lyg.framework.service.IUserService;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.DefaultParameterNameDiscoverer;
import org.springframework.expression.EvaluationContext;
import org.springframework.expression.Expression;
import org.springframework.expression.common.TemplateParserContext;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

@Aspect
@Component
@Slf4j
public class OpLogAspect {

    @Autowired
    IOpLogService opLogService;

    @Autowired
    IUserService userService;

    @Autowired
    HttpServletRequest request;

    /**
     * 用于SpEL表达式解析.
     */
    private static SpelExpressionParser parser = new SpelExpressionParser();
    /**
     * 用于获取方法参数定义名字.  能根据你传入的方法 获取该方法的参数名
     */
    private static DefaultParameterNameDiscoverer nameDiscoverer = new DefaultParameterNameDiscoverer();
    // 需要被SpEl解析的模板前缀和后缀 {{ expression  }}
    private static final TemplateParserContext TEMPLATE_PARSER_CONTEXT = new TemplateParserContext("{{", "}}");

    /**
     * 切点
     */
    @Pointcut("@annotation(com.lyg.framework.log.OpLog)")
    public void opLogPointCut() {}

    @AfterReturning(value = "opLogPointCut()", returning = "jsonResult")
    public void doAfterReturning(JoinPoint joinPoint, Object jsonResult) {
        try {
            Object user = request.getAttribute("user");
            String ip = IpUtil.getIpAddr(request);
            String ua = request.getHeader("User-Agent");
//            String tenantId = (String) request.getAttribute("tenantId");

            Map<String, Object> data = new HashMap<>();
            data.put("uri", request.getRequestURI());
            data.put("ip", ip);
            data.put("ua", ua);
            data.put("user", user);
            //
            MethodSignature methodSignature = (MethodSignature) joinPoint.getSignature();
            Method method = methodSignature.getMethod();
            OpLog opLog = method.getAnnotation(OpLog.class);
            if (opLog != null) {
                data.put("module", opLog.module());
                data.put("action", opLog.action());
                if (null != opLog.desc() && !"".equals(opLog.desc())) {
                    data.put("desc", generateKeyBySpEL(opLog.desc(), joinPoint));
                }
            }

            opLogService.save(data);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private String generateKeyBySpEL(String spELString, JoinPoint joinPoint) {
        // 通过joinPoint获取被注解方法
        MethodSignature methodSignature = (MethodSignature) joinPoint.getSignature();
        Method method = methodSignature.getMethod();
        // 使用spring的DefaultParameterNameDiscoverer获取方法形参名数组
        String[] paramNames = nameDiscoverer.getParameterNames(method);
        // 解析过后的Spring表达式对象
        Expression expression = parser.parseExpression(spELString, TEMPLATE_PARSER_CONTEXT);
        // spring的表达式上下文对象
        EvaluationContext context = new StandardEvaluationContext();
        // 通过joinPoint获取被注解方法的形参
        Object[] args = joinPoint.getArgs();
        // 给上下文赋值
        for (int i = 0; i < args.length; i++) {
            context.setVariable(paramNames[i], args[i]);
        }
        // 表达式从上下文中计算出实际参数值
        return expression.getValue(context).toString();
    }

}
